<?php 
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html

require_once "approach/astronomy/solarsystem/SolarSystemConstants.php";

/****************************************************************************************
Computations related to moon phases (new / full moon, quarters).

@author   <a href="http://www.tig12.net">Thierry Graff</a>
@history 2003.01.25 : Creation from a split of astronomy.php
@history	: 2008.08.14 04:53 : Put in a PHP5 class
@todo unit test
****************************************************************************************/

class MoonPhases{
  
  //********************* getPhase ******************************
  /**
    Computes the phase of the moon, from Moon and Sun longitudes
    @return The pahse, in decimal degrees
    @history 2008.08.14 05:53 : Creation
  */
  public static function getPhase($sunLong, $moonLong){
    $phase = uMaths::mod360($moonLong - $sunLong);
  return $phase;
  }// end getPhase
  
  
  //********************* getQuarterDate ******************************
  /**
    Computes the date of the moon quarter before or after a given date.
    From Meeus, Astronomical Algorithms, 2nd edition, chap. 49.
    @param $jd Julian day
    @param $quarter The required moon quarter, using a constant of SolarSystemConstants.php.
    @param $beforeAfter A string indicating if the required phase should be searched
                  after or before $jd. Possible values : 'before' or 'after'.
    @return The julian day of the desired event
    @history 2003.01.15 Creation from OCR (Optical Character Recognition) of Meeus' book.
  */
  public static function getQuarterDate($jd, $quarter, $beforeAfter){
    require_once "spacetime/Date.php";
    // parameters
    $beforeAfter = strToLower($beforeAfter);
    list($year, $month, $day, $hour, $min, $sec) = Date::jdToDate($jd);
    $decYear = Date::decimalYear($year, $month, $day, $hour, $min, $sec);
    $k = ($decYear - 2000) * 12.3685;
    $decK = $k - floor($k); // decimal part of $k
//echo "<br/>k = $k  -  decK = $decK";
    // Adapt $k according to $quarter and $afterBefore
    switch($quarter){
      case SolarSystemConstants::NEW_MOON : $shift = 0.0; break;
      case SolarSystemConstants::FIRST_MOON_QUARTER : $shift = 0.25; break;
      case SolarSystemConstants::FULL_MOON : $shift = 0.5; break;
      case SolarSystemConstants::LAST_MOON_QUARTER : $shift = 0.75; break;
    }
    if($beforeAfter == "before"){
      if($decK <= $shift) $k = floor($k) - 1 + $shift;
      else $k = floor($k) + $shift;
    }
    else{ // "after"
      if($decK <= $shift) $k = floor($k) + $shift;
      else $k = floor($k) + 1 + $shift;
    }
  //echo "<br/>k = $k";
    $T = $k/1236.85;
  //echo "<br/>T = $T";
    $T2 = $T*$T;
    $T3 = $T2*$T;
    $T4 = $T3*$T;
    // jd for the mean phase.
    $JDE = 2451550.09766 + 29.530588861*$k
                         + 0.00015437*$T2
                         - 0.000000150*$T3
                         + 0.00000000073*$T4;
  //echo "<br/>JDE = $JDE";
    $E = 1 - 0.002516*$T - 0.0000074*$T2; // formula 47.6
    $E2 = $E*$E;
  //echo "<br/>E = $E";
    $M = deg2rad(uMaths::mod360(2.5534	+ 29.10535670*$k
                        - 0.0000014*$T2
                        - 0.00000011*$T3));
    $Mp = deg2rad(uMaths::mod360(201.5643 + 385.81693528*$k
                         + 0.0107582*$T2
                         + 0.00001238*$T3
                         - 0.000000058*$T4));
    $F = deg2rad(uMaths::mod360(160.7108 + 390.67050284*$k
                        - 0.0016118*$T2
                        - 0.00000227*$T3
                        + 0.000000011*$T4));
    $om = deg2rad(uMaths::mod360(124.7746 - 1.56375588*$k
                         + 0.0020672*$T2
                         + 0.00000215*$T3));
    // Planetary arguments, in degrees.
    $A1  = deg2rad(uMaths::mod360(299.77 + 0.107408*$k - 0.009173*$T2));
    $A2  = deg2rad(uMaths::mod360(251.88 + 0.016321*$k));
    $A3  = deg2rad(uMaths::mod360(251.83 + 26.651886*$k));
    $A4  = deg2rad(uMaths::mod360(349.42 + 36.412478*$k));
    $A5  = deg2rad(uMaths::mod360( 84.66 + 18.206239*$k));
    $A6  = deg2rad(uMaths::mod360(141.74 + 53.303771*$k));
    $A7  = deg2rad(uMaths::mod360(207.14 + 2.453732*$k));
    $A8  = deg2rad(uMaths::mod360(154.84 + 7.306860*$k));
    $A9  = deg2rad(uMaths::mod360( 34.52 + 27.261239*$k));
    $A10 = deg2rad(uMaths::mod360(207.19 + 0.121824*$k));
    $A11 = deg2rad(uMaths::mod360(291.34 + 1.844379*$k));
    $A12 = deg2rad(uMaths::mod360(161.72 + 24.198154*$k));
    $A13 = deg2rad(uMaths::mod360(239.56 + 25.513099*$k));
    $A14 = deg2rad(uMaths::mod360(331.55 + 3.592518*$k));
    //
    // Corrections (in days) to get the true phase
    switch($quarter){
      case SolarSystemConstants::NEW_MOON :
        $corr = 
        - 0.40720       * sin($Mp)
        + 0.17241 * $E  * sin($M)
        + 0.01608       * sin(2*$Mp)
        + 0.01039       * sin(2*$F)
        + 0.00739 * $E  * sin($Mp - $M)
        - 0.00514 * $E  * sin($Mp + $M)
        + 0.00208 * $E2 * sin(2*$M)
        - 0.00111       * sin($Mp - 2*$F)
        - 0.00057       * sin($Mp + 2*$F)
        + 0.00056 * $E  * sin(2*$Mp + $M)
        - 0.00042       * sin(3*$Mp)
        + 0.00042 * $E  * sin($M + 2*$F)
        + 0.00038 * $E  * sin($M - 2*$F)
        - 0.00024 * $E  * sin(2*$Mp - $M)
        - 0.00017       * sin($om)
        - 0.00007       * sin($Mp + 2*$M)
        + 0.00004       * sin(2*$Mp - 2*$F)
        + 0.00004       * sin(3*$M)
        + 0.00003       * sin($Mp+ $M - 2*$F)
        + 0.00003       * sin(2*$Mp + 2*$F)
        - 0.00003       * sin($Mp + $M + 2*$F)
        + 0.00003       * sin($Mp - $M + 2*$F)
        - 0.00002       * sin($Mp - $M - 2*$F)
        - 0.00002       * sin(3*$Mp + $M)
        + 0.00002       * sin(4*$Mp);
      break;
      case SolarSystemConstants::FULL_MOON :
        $corr =
        - 0.40614	      * sin($Mp)
        + 0.17302 * $E  * sin($M)
        + 0.01614       * sin(2*$Mp)
        + 0.01043       * sin(2*$F)
        + 0.00734 * $E  * sin($Mp - $M)
        - 0.00515 * $E  * sin($Mp + $M)
        + 0.00209 * $E2	* sin(2*$M)
        - 0.00111	      * sin($Mp - 2*$F)
        - 0.00057	      * sin($Mp + 2*$F)
        + 0.00056 * $E  * sin(2*$Mp + $M)
        - 0.00042	      * sin(3*$Mp)
        + 0.00042 * $E  * sin($M + 2*$F)
        + 0.00038 * $E  * sin($M - 2*$F)
        - 0.00024 * $E  * sin(2*$Mp - $M)
        - 0.00017	      * sin($om)
        - 0.00007	      * sin($Mp + 2*$M)
        + 0.00004	      * sin(2*$Mp - 2*$F)
        + 0.00004	      * sin(3*$M)
        + 0.00003	      * sin($Mp + $M - 2*$F)
        + 0.00003	      * sin(2*$Mp + 2*$F)
        - 0.00003	      * sin($Mp + $M + 2*$F)
        + 0.00003	      * sin($Mp - $M + 2*$F)
        - 0.00002	      * sin($Mp - $M - 2*$F)
        - 0.00002	      * sin(3*$Mp + $M)
        + 0.00002	      * sin(4*$Mp);
      break;
      default : //First and last quarter
        $corr =
        - 0.62801       * sin($Mp)
        + 0.17172 * $E  * sin($M)
        - 0.01183 * $E  * sin($Mp +$M)
        + 0.00862       * sin(2*$Mp)
        + 0.00804       * sin(2*$F)
        + 0.00454 * $E  * sin($Mp - $M)
        + 0.00204 * $E2 * sin(2*$M)
        - 0.00180       * sin($Mp - 2*$F)
        - 0.00070       * sin($Mp + 2*$F)
        - 0.00040       * sin(3*$Mp)
        - 0.00034 * $E  * sin(2*$Mp - $M)
        + 0.00032 * $E  * sin($M + 2*$F)
        + 0.00032 * $E  * sin($M - 2*$F)
        - 0.00028 * $E2 * sin($Mp + 2*$M)
        + 0.00027 * $E  * sin(2*$Mp + $M)
        - 0.00017       * sin($om)
        - 0.00005       * sin($Mp- $M - 2*$F)
        + 0.00004       * sin(2*$Mp + 2*$F)
        - 0.00004       * sin($Mp +$M+2*$F)
        + 0.00004       * sin($Mp- 2*$M)
        + 0.00003       * sin($Mp + $M - 2*$F)
        + 0.00003       * sin(3*$M)
        + 0.00002       * sin(2*$Mp - 2*$F)
        + 0.00002       * sin($Mp- $M + 2*$F)
        - 0.00002       * sin(3*$Mp + $M);
      break;
    }// end switch
  //echo "<br/>corr = $corr";
    //
    // Additional corrections :
    $addCorr = 0;
    if($quarter == SolarSystemConstants::FIRST_MOON_QUARTER || $quarter == SolarSystemConstants::LAST_MOON_QUARTER){
      $W = 0.00306 - 0.00038*$E*cos($M) + 0.00026*cos($Mp)
         - 0.00002*cos($Mp - $M) + 0.00002*cos($Mp + $M) + 0.00002*cos(2*$F);
      if($quarter == SolarSystemConstants::FIRST_MOON_QUARTER) $addCorr = $W;
      if($quarter == SolarSystemConstants::LAST_MOON_QUARTER) $addCorr = -$W;
    }
  //    $addCorr = 0;
  //echo "<br/>addCorr = $addCorr";
    $addCorr +=
      0.000325 * sin($A1)
    + 0.000165 * sin($A2)
    + 0.000164 * sin($A3)
    + 0.000126 * sin($A4)
    + 0.000110 * sin($A5)
    + 0.000062 * sin($A6)
    + 0.000060 * sin($A7)
    + 0.000056 * sin($A8)
    + 0.000047 * sin($A9)
    + 0.000042 * sin($A10)
    + 0.000040 * sin($A11)
    + 0.000037 * sin($A12)
    + 0.000035 * sin($A13)
    + 0.000023 * sin($A14);
    return $JDE + $corr + $addCorr;
  }// end getQuarterDate

  
  //********************* getAge ******************************
  /**
    Computes the age of the moon (time elapsed since last new moon).
    @param $jd Julian day 
    @return The age of the moon, in decimal days
    @history 2008.09.19 15:38 : Creation
  */
  public static function getAge($jd){
    $jdNew = self::getQuarterDate($jd, SolarSystemConstants::NEW_MOON, 'before');
    return $jd - $jdNew;
  }// end getAge
  
  
  //********************* getLunationDates ******************************
  /**
    Comuptes the quarter dates of the current lunation at a given date.
    @param $jd Julian day
    @return An array containing the julian days of quarter dates, from previous to next new moon
    @history 2008.09.19 16:57 : Creation from an old function
  */
  public static function getLunationDates($jd){
    $jds[0] = self::getQuarterDate($jd, SolarSystemConstants::NEW_MOON, "before");
    $jds[1] = self::getQuarterDate($jds[0], SolarSystemConstants::FIRST_MOON_QUARTER, "after");
    $jds[2] = self::getQuarterDate($jds[1], SolarSystemConstants::FULL_MOON, "after");
    $jds[3] = self::getQuarterDate($jds[2], SolarSystemConstants::LAST_MOON_QUARTER, "after");
    $jds[4] = self::getQuarterDate($jds[3], SolarSystemConstants::NEW_MOON, "after");
    return $jds;
  }// end getLunationDates
  
  
  //********************* getPhaseImage ******************************
  /** 
    Computes an image of the moon's phase.
    Depends on the existence of a directory containing images of full moon, filling these requirements :
    - Images must be squares fitting exactly the full moon circle.
    - Images must be named "fullMoon_<i>size</i>.jpg" (ex : fullMoon_100.jpg)
    @param $sunLong Longitude of the Sun.
    @param $moonLong Longitude of the Moon.
    @param $options Associative array of key / value pairs. Keys can be :
                  'imageDir' - compulsory - relative path from the calling page to the directory containing full moon images
                  'imageSize' - compulsory - Size of the image, in pixels. Supported sizes : 50, 80, 100, 150, 200
    @return An image object
    @throw Exception if invalid or missing parameter
    @history 2003.01.11 : Creation
    @history 2008.09.19 13:51 : Port to PHP5
  */
  public static function getPhaseImage($sunLong, $moonLong, $options){
    // parameters
    if(!isset($options['imageSize'])) throw new Exception("Missing parameter : \$options['imageSize']");
    if(!in_array($options['imageSize'], array(50, 80, 100, 150, 200))) throw new Exception("Invalid parameter : \$options['imageSize'] ; {$options['imageSize']} not supported");
    $s = $options['imageSize'];
    if(!isset($options['imageDir'])) throw new Exception("Missing parameter : \$options['imageDir']");
    //
    $phase = self::getPhase($moonLong, $sunLong);
    $k = (1 + cos(deg2rad($phase))) / 2; // from Meeus, chap. 31
    $s2 = $s/2;
    $img = @imageCreate($s, $s);
    if($img === false) throw new Exception ("Cannot Initialize new GD image");
    $white = ImageColorAllocate ($img, 255, 255, 255);
    $black = ImageColorAllocate ($img, 0, 0, 0);
    $imageFileName = $options['imageDir'] . "/fullMoon_$s.jpg"; 
    ImageCopy($img, imageCreateFromJpeg($imageFileName), 0, 0, 0, 0, $s, $s);
    // Fill the full moon image with black to simulate the phase.
    if($phase <= 90){
      imagefilledrectangle($img, 0, 0, $s2, $s, $black);
      $lim = $s*(2*$k - 1);
      for($i = 0; $i < $lim; $i++){
        imageArc($img, $s2, $s2, $i, $s, 270, 90, $black);
      }
    }
    else if($phase <= 180){
      $lim = -$s*(2*$k - 1);
      for($i = $s; $i > $lim; $i--){
        imageArc($img, $s2, $s2, $i, $s, 90, 270, $black);
      }
    }
    else if($phase <= 270){
      $lim = -$s*(2*$k - 1);
      for($i = $s; $i > $lim; $i--){
        imageArc($img, $s2, $s2, $i, $s, 270, 90, $black);
      }
    }
    else{ // $phase > 270
      imagefilledrectangle($img, $s2, 0, $s, $s, $black);
      $lim = $s*(2*$k - 1);
      for($i = 0; $i < $lim; $i++){
        imageArc($img, $s2, $s2, $i, $s, 90, 270, $black);
      }
    }
    return $img;
  }// end getPhaseImage
  
  
  //********************* zzz_getLunationImage ******************************
  /**
    For a given date, returns an image with the current date and 
    the dates of quarters of the current lunation.
    @param $date The current date, formatted "YYYY-MM-DD" (any separator can fit).
    @param $time The current time, formatted "HH-MM-SS" (any separator can fit).
    @param $qImgPath Path to the directory containing the images of lunar phases.
            <BR>The images must be called "new.png", "first.png", "full.png" and "last.png".
            <BR>The images must be squares and have all the same size.
    @param $labels Array containing the labels displayed on the image. Contains :
                   - The title,
                   - a word meaning "age"
                   - a word meaning "days"
    @param $lang The language used to format the dates.
    @history 2003.01.23 Drawing of the image.
  */
  public static function getLunationImage($date, $time, $qImgPath, $labels, $lang){
    $qImgNames = array("icon-new.png", "icon-first.png", 
                       "icon-full.png", "icon-last.png", "icon-new.png");
    $qis = 16; // quarter image size - could be computed (create one image and get its size).
    $tW = 4; $tH = 8; // size of the triangle marking the current day
    $titleFont = 5;
    $ageFont = 3;
    $dateFont = 2;
    $hourFont = 2;
    $margin = 10;
    $gradHa = 5; // Size of graduations above the line
    $gradHb = 5; // Size of graduations below the line
    $gap1 = 15; // between the title and the top of quarter images
    $gap2 = 4; // between the graduation and the quarter image
    $gap3 = 2; // between the date and the time
    $gap4 = 20; // between 2 quarters (images or date strings)
    // Compute the quarter dates
    $dateTime = dt_sqlToArray($date, $time);
    $jd = dateToJd($dateTime);
    $jds[0] = self::getQuarterDate(SolarSystemConstants::NEW_MOON, $jd, "before");
    $jds[1] = self::getQuarterDate(SolarSystemConstants::FIRST_MOON_QUARTER, $jds[0], "after");
    $jds[2] = self::getQuarterDate(SolarSystemConstants::FULL_MOON, $jds[1], "after");
    $jds[3] = self::getQuarterDate(SolarSystemConstants::LAST_MOON_QUARTER, $jds[2], "after");
    $jds[4] = self::getQuarterDate(SolarSystemConstants::NEW_MOON, $jds[3], "after");
    // Format these dates
    for($i = 0; $i < 5; $i++){
      list($qDates[$i], $qTimes[$i]) = dt_arrayToDisplay(jdToDate($jds[$i]), $lang, "MD", "HM");
    }
    // Age of the moon
    $age = formatNumber($jd - $jds[0], 2);
    $strAge = $labels[1] . " : " . $age . " " . $labels[2];
    // Prepare the image.
    list($dateW, $dateH) = getStringSize("30/08", $dateFont);
    list($titleW, $titleH) = getStringSize($labels[0], $dateFont);
    list($ageW, $ageH) = getStringSize($strAge, $ageFont);
    //
    $imgW = 350;
    $imgH = 120;
    $img = @imageCreate($imgW, $imgH)
           or die ("<B>astronomy.php - moonLunationImage</B> - Cannot Initialize new GD image");
    $white = ImageColorAllocate ($img, 255, 255, 255);
    $black = ImageColorAllocate ($img, 0, 0, 0);
    // Draw the title
    imageString($img, $titleFont, ($imgW-$titleW)/2, $margin, $labels[0], $black);
    // Draw the main line 
    $lineY = $margin + $titleH + $gap1 + $qis + $gap2 + $gradHa;
    $xI = $margin + max($qis, $dateW)/2; // Initial
    $xF = $imgW - $xI; // Final
    imageLine($img, $xI, $lineY, $xF, $lineY, $black);
    //Draw the quarter images, the graduations and the dates
    $offset = ($xF - $xI)/4;
    for($i = 0; $i < 5; $i++){
      $x = $xI + $offset * $i;
      imageLine($img, $x, $lineY + $gradHb, $x, $lineY - $gradHa, $black); // graduation
      $qImgName = $qImgPath . "/" . $qImgNames[$i];
      ImageCopy($img, imageCreateFromPng($qImgName), 
                $x-$qis/2, $lineY-$gradHa-$qis-$gap2, 0, 0, $qis, $qis);
      $y = $lineY + $gradHb;
      imageString($img, $dateFont, $x-$dateW/2, $y, $qDates[$i], $black);
      $y += $dateH + $gap3;
      imageString($img, $hourFont, $x-$dateW/2, $y, $qTimes[$i], $black);
    }
    // Draw the mark for actual date
    $x = ($xF - $xI) * ($jd - $jds[0]) / ($jds[4] - $jds[0]);
    imageLine($img, $x, $lineY, $x-$tW, $lineY-$tH, $black);
    imageLine($img, $x-$tW, $lineY-$tH, $x+$tW, $lineY-$tH, $black);
    imageLine($img, $x+$tW, $lineY-$tH, $x, $lineY, $black);
    imageString($img, $ageFont, $margin, $imgH-$margin-$ageH, $strAge, $black);
    return $img;
  }// end getLunationImage
  
  
}// end class
?>