<?php 
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html
/****************************************************************************************
Contains mathematical utilities.

@license  GPL
@author   <a href="http://www.tig12.net">Thierry Graff</a>
@history 2002.10.10 : Creation
@history 2008.05.07 : Transformed to a class
****************************************************************************************/

class uMaths{
  
  //**************************************************************************
  //                            Constants
  //**************************************************************************
  /** Constant with a value of PI / 180 */
  const PIs180 =0.017453292519943295769236907684886;
  
  /** Constant whose value is ln(10). */
  const ln10 = 2.30258509299404568401799145468436;
  
  /** Constant to convert arc seconds to radians, whose value is <FONT FACE="symbol">p</FONT> / (180 * 3600). */
  const ARCSEC_TO_RAD = 4.84813681109535993589914102357948e-6;
  
  /** Constant to convert radians to arc seconds, whose value is 180 * 3600 / <FONT FACE="symbol">p</FONT>. */
  const RAD_TO_ARCSEC = 206264.806247096355156473357330779;
  
  // ******* Constants to desingate coordinates. ********
  /** Constant designating the first spatial coordinate (<B>X</B> in cartesian, 
  <B><FONT FACE="Symbol">r</FONT></B> in spherical). */
  const COORD_X0 = 0;
  /** Constant designating the second spatial coordinate (<B>Y</B> in cartesian,
  <B><FONT FACE="Symbol">q</FONT></B> in spherical). */
  const COORD_X1 = 1;
  /** Constant designating the third spatial coordinate
  (<B>Z</B> in cartesian, <B><FONT FACE="Symbol">j</FONT></B> in spherical). */
  const COORD_X2 = 2;
  
  //**************************************************************************
  //                            Public methods
  //**************************************************************************
  
  //********************* sphereToCart ******************************
  /** Transform a Vector3 from spherical to cartesian expression.
  @param $v A Vector3, with the angles expressed in degrees.
  @return The cartesian coordinates ; the unit is the same as the distance unit of $v.
  */
  public static function sphereToCart($v){
    $r = $v->x0;
    $theta = deg2rad($v->x1);
    $phi = deg2rad($v->x2);
    $x = $r*cos($phi)*cos($theta);
    $y = $r*cos($phi)*sin($theta);
    $z = $r*sin($phi);
    return new Vector3($x, $y, $z);
  } // end sphereToCart
  
  
  //********************* cartToSphere ******************************
  /** Transform a Vector3 from  cartesian to spherical expression.
  @param $v A Vector3, with the angles expressed in degrees.
  @return The spherical coordinates ; the angles are in degrees.
  */
  public static function cartToSphere($v){
    // variables to remember initial values.
    $X = $v->x0;
    $Y = $v->x1;
    $Z = $v->x2;
    $rho2 = $X*$X + $Y*$Y + $Z*$Z;
    $rho = sqrt($rho2);
    $theta = self::atan3($Y, $X);
    $phi = asin($Z / $rho);
    return new Vector3($rho, rad2deg($theta), rad2deg($phi));
  } // end cartToSphere
  
  
  //*************************** atan3 ***********************************
  /** Computation of arcTangent, giving a result in [0, 2*<FONT FACE="symbol">p</FONT>[.
  @return A number <CODE>alpha</CODE> such as <CODE>cos(alpha) = x/sqrt(x*x + y*y)</CODE> and 
  <CODE>sin(alpha) = y/sqrt(x*x + y*y)</CODE> and alpha belongs to [0, 2*<FONT FACE="symbol">p</FONT>[.
  */
  public static function atan3($y, $x){
    $alpha = atan2($y, $x); // belongs to [-PI, PI[
    if ($alpha >= 0) return $alpha;
    else             return $alpha + 2*M_PI;
  } // end atan3
  
  
  //*********************************************************
  //                Modulo functions
  //*********************************************************
  
  //********************* mod360 ******************************
  /** Returns a number modulo 360 (between 0 and 360). */
  public static function mod360($nb){
    if($nb >= 0){
      $dec = $nb - floor($nb);
      return $nb%360 + $dec;
    }
    else{
      $dec = $nb - floor($nb);
      if($dec != 0) $dec -= 1;
      return $nb%360 + $dec + 360;
    }
  } // end mod360
  
  
  //*************************** modX() ***************************************
  /** Reduction of a number in the interval [0, limit].
  @param nb any number.
  @param limit any positive number.
  @return <CODE>nb</CODE> modulo <CODE>limit</CODE>.
  */
  public static function modX($nb, $limit){
    while ($nb > $limit) $nb -= $limit;
    while ($nb < 0.0) $nb += $limit;
    return $nb;
  }// end modX
  public static function modX2($nb, $limit){
    if($nb >= 0){
      $dec = $nb - floor($nb);
      return $nb%$limit + $dec;
    }
    else{
      $dec = $nb - floor($nb);
      if($dec != 0) $dec -= 1;
      return $nb%$limit + $dec + $limit;
    }
  } //end modX()
  
  
  //********************* sub360 ******************************
  /** Performs a substraction modulo 360 : computes $a2 - $a1, the angle between $a1 and $a2.
  <BR>Ex : sub360(355, 10) = 345 ; sub360(10, 355) = 15.
  */
  public static function sub360($a2, $a1){
    if($a2 >= $a1) return $a2 - $a1;
    else return $a2 + 360 - $a1;
  } // end sub360
  
  
  //********************* subX ******************************
  /** Performs a substraction modulo X : computes $a2 - $a1.
  <BR>Ex : subX(365, 24, 28) = 361 ; sub(365, 28, 24) = 4.
  <BR>Warning : parameter $a2 is first.
  */
  public static function subX($modulo, $a2, $a1){
    if($a2 >= $a1) return $a2 - $a1;
    else return $a2 + $modulo - $a1;
  } // end subX
  
  
}// end class
?>