<?php 
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html
/****************************************************************************************
Functions to format strings.

@license  GPL
@author   <a href="http://www.tig12.net">Thierry Graff</a>
@history 2002.10.10 : Creation
@history 2007.12.15 : Transformed to a class
@history 2008.01.11 11:38 : Placed in new framework
@history 2008.03.07 : Added number_format()

****************************************************************************************/
class uFormatStrings{
  
  //******************** addZeroes *******************************
  /**
    Formats a positive number, considered as a string.
    Adds zeroes in front of $nb to get a string of length $size.
    Ex : <code>addZeroes(92, 4)</code> returns "0092".
    @param $size : The size the returned number must have.
  */
  public static function addZeroes($nb, $size="2"){
    $l = strlen($nb);
    if($size >= $l) return str_repeat("0", $size - $l) . $nb;
    else return $nb;
  }// end addZeroes
  
  
  //******************** toDMS *******************************
  /** 
    Formats a decimal degree to a string : <code>ddd&deg;mm'ss</code>.
    @param $x The number to format
    @param $options Associative array of key /value pairs ; keys can be :
                  - 'format' : can be 'D', 'DM', or 'DMS', to have a string with degrees only, or degrees minutes, or degrees minutes seconds
                  Default : 'DMS'
                  - 'echo_last_symbol' : Boolean ; If true the last symbol (° for degrees ; ' for minutes ; " for seconds) is appended to the string
                  ex with format = 'DM', and $x = 12.5 : returns <code>12°30'</code> if true ; returns <code>12°30</code> if false
                  Default : true
                  - 'degree_symbol' : The symbol used for degrees (in general '°' or '&amp;deg;')
                  Default : '°'
                  - 'reduce' : Boolean ; if true, $x is first reduced to interval [0, 360]
                  Default : false
    @throw IllegalArgumentException if $options['format'] is not correct
    */
  function toDMS($x, $options = array()){
    // parameter stuff
    if(!isset($options['format'])) $options['format'] = 'DMS';
    if(!isset($options['echo_last_symbol'])) $options['echo_last_symbol'] = true;
    if(!isset($options['degree_symbol'])) $options['degree_symbol'] = '°';
    if(!isset($options['reduce'])) $options['reduce'] = false;
    if(!in_array($options['format'], array('D', 'DM', 'DMS')))
      throw new IllegalArgumentException("Illegal parameter \$options['format'] : {$options['format']}");
    //
    if($options['reduce']) $x = uMaths::mod360($x); 
    $pInt = ($x>0) ? floor($x) : ceil($x); // integer part of x
    if($options['format'] == 'D')
      return $pInt . ($options['echo_last_symbol'] ? $options['degree_symbol'] : '');
    $pDec = $x - $pInt;
    $nbMin = floor($pDec*60); // nb of minuts in pDec
    if($options['format'] == 'DM')
      return $pInt . $options['degree_symbol'] . ' ' . self::addZeroes($nbMin) . ($options['echo_last_symbol'] ? "'" : '');
    $nbSec = floor($pDec*3600)-60*$nbMin; //nb of seconds in pDec
    return $pInt . $options['degree_symbol'] . ' ' . self::addZeroes($nbMin) . "' " . self::addZeroes($nbSec) . ($options['echo_last_symbol'] ? '"' : '');
   }// end toDMS
   
   
  //******************** number_format *******************************
  /**
    Equivalent to php function <code>number_format($nb, $dec, '.', ' ')</code>
    Ex : 12345.111 gives "12 345.111"
    The format is a mix of english (for decimal separator) and french notation (for thousands separator).
    @param $nb The number to format
    @param $dec The number of decimals
    @return String The formatted number
  */
  public static function number_format($nb, $dec=0){
    return number_format($nb, $dec, '.', ' ');
  }// end number_format
  
  
  //******************** dirify *******************************
  /**
    Transforms an ISO-8859-1 or UTF8 string to a string usable in a file name.
    Adaptation from a script found at http://www.phpit.net/code/dirify/,  2005.12.22 02:24
    @param $str : The string to dirify.
    @param $charset Can be 'utf8' or 'iso-8859-1'
    @param $replace String Replacement character (or string) for space characters
    @return The transformed string
  */
  public static function dirify($str, $charset='utf8', $replace='-') {
    if($charset == 'utf8') $str = utf8_decode($str);
    $str = self::convert_high_ascii($str);  ## convert high-ASCII chars to 7bit.
    $str = strtolower($str);           ## lower-case.
    $str = strip_tags($str);       ## remove HTML tags.
    $str = preg_replace('!&#038;[^;\s]+;!','',$str);         ## remove HTML entities.
    $str = preg_replace('![^\w\s]!',$replace,$str);           ## remove non-word/space chars.
    $str = preg_replace('!\s+!', $replace, $str);               ## change space chars to $replace
    // remove multiple $replace
    while(strPos($str, "{$replace}{$replace}") !== false){
      $str = str_replace("{$replace}{$replace}", $replace, $str);
    }
    // if last character is replace, remove it
    if(substr($str, -1) == $replace)
      $str = substr($str, 0, -1);
    return $str;    
  }// end dirify
  
  
  //******************** htmlEntities *******************************
  /**
    Useful only for php < version 5.3
    Does the same as php function htmlentities, but doesn't double encode
    For parameters, see php function htmlentities.
    @todo : test charset
  */
  public static function htmlentities($string, $quote_style=ENT_NOQUOTES){
    $trans = get_html_translation_table(HTML_ENTITIES, $quote_style);
    unset($trans['<']);
    unset($trans['>']);
    unset($trans['&']);
//    unset($trans['"']);
//echo "<pre>"; print_r($trans); echo "</pre>";
    $string = utf8_decode($string);
    $encoded = strtr($string, $trans);
    return $encoded;
  }// end htmlEntities
  
  
  
  // *******************************************************************
  //                                                   PRIVATE
  // *******************************************************************
  
  //******************** convert_high_ascii *******************************
  /** Auxiliary function of dirify */
  private static function convert_high_ascii($s){
       $HighASCII = array(
           "!\xc0!" => 'A',    # A`
           "!\xe0!" => 'a',    # a`
           "!\xc1!" => 'A',    # A'
           "!\xe1!" => 'a',    # a'
           "!\xc2!" => 'A',    # A^
           "!\xe2!" => 'a',    # a^
           "!\xc4!" => 'Ae',   # A:
           "!\xe4!" => 'ae',   # a:
           "!\xc3!" => 'A',    # A~
           "!\xe3!" => 'a',    # a~
           "!\xc8!" => 'E',    # E`
           "!\xe8!" => 'e',    # e`
           "!\xc9!" => 'E',    # E'
           "!\xe9!" => 'e',    # e'
           "!\xca!" => 'E',    # E^
           "!\xea!" => 'e',    # e^
           "!\xcb!" => 'Ee',   # E:
           "!\xeb!" => 'ee',   # e:
           "!\xcc!" => 'I',    # I`
           "!\xec!" => 'i',    # i`
           "!\xcd!" => 'I',    # I'
           "!\xed!" => 'i',    # i'
           "!\xce!" => 'I',    # I^
           "!\xee!" => 'i',    # i^
           "!\xcf!" => 'Ie',   # I:
           "!\xef!" => 'ie',   # i:
           "!\xd2!" => 'O',    # O`
           "!\xf2!" => 'o',    # o`
           "!\xd3!" => 'O',    # O'
           "!\xf3!" => 'o',    # o'
           "!\xd4!" => 'O',    # O^
           "!\xf4!" => 'o',    # o^
           "!\xd6!" => 'Oe',   # O:
           "!\xf6!" => 'oe',   # o:
           "!\xd5!" => 'O',    # O~
           "!\xf5!" => 'o',    # o~
           "!\xd8!" => 'Oe',   # O/
           "!\xf8!" => 'oe',   # o/
           "!\xd9!" => 'U',    # U`
           "!\xf9!" => 'u',    # u`
           "!\xda!" => 'U',    # U'
           "!\xfa!" => 'u',    # u'
           "!\xdb!" => 'U',    # U^
           "!\xfb!" => 'u',    # u^
           "!\xdc!" => 'Ue',   # U:
           "!\xfc!" => 'ue',   # u:
           "!\xc7!" => 'C',    # ,C
           "!\xe7!" => 'c',    # ,c
           "!\xd1!" => 'N',    # N~
           "!\xf1!" => 'n',    # n~
           "!\xdf!" => 'ss'
       );
       $find = array_keys($HighASCII);
       $replace = array_values($HighASCII);
       $s = preg_replace($find,$replace,$s);
       return $s;
  }// end convert_high_ascii

}// end class uFormatStrings
?>